<?php

/**
 * SI_Processor_Limits Controller
 *
 * @package Sprout_Invoice
 * @subpackage SI_Processor_Limits
 */
class SI_Processor_Limits_Settings extends SI_Processor_Limits {

	public static function init() {

		// settings
		add_filter( 'si_payment_settings', array( __CLASS__, 'register_options' ) );
		add_filter( 'si_settings', array( __CLASS__, 'register_addon_settings' ) );

	}

	/**
	 * Hooked on init add the settings page and options.
	 *
	 */
	public static function register_options( $settings = array() ) {
		// Settings

		$enabled_gateways = SI_Payment_Processors::enabled_processors();
		$payment_gateways = SI_Payment_Processors::get_registered_processors();

		if ( empty( $enabled_gateways ) ) {
			$settings['si_processor_limits_settings'] = array(
				'title'       => esc_html__( 'Processor Limits', 'sprout-invoices' ),
				'weight'      => 250,
				'description' => sprintf(
					// translators: 1: break and enable processor button 2: v-on open 3: v-on close 4: v-bind open 5: v-bind element 6: v-bind value 7: button text 8: closing button tag.
					__( 'In order to use processor limits add-on you will need to enable a Processor first.%1$s%2$s%3$s%4$s%5$s%6$s%7$s%8$s', 'sprout-invoices' ),
					'<br><a class="button button-primary" id="si_enable_processor" href="#payment-processors"',
					' v-on:click="makeTabActive(',
					"'payment-processors')",
					'" v-bind:class="{ active : isActiveTab(',
					"'payment-processors')",
					'== true }">',
					__( 'Enable a Processor', 'sprout-invoices' ),
					'</a>',
				),
			);
		}

		foreach ( $payment_gateways as $class => $label ) {
			if ( ! in_array( $class, $enabled_gateways ) ) {
				continue;
			}
			if ( ! method_exists( $class, 'get_instance' ) ) {
				continue;
			}
			$payment_processor = call_user_func( array( $class, 'get_instance' ) );

			$settings[ 'si_processor_limits_settings' . $class ] = array(
				'title'    => sprintf( __( '%s Limits', 'sprout-invoices' ), $label ),
				'weight'   => 250,
				'settings' => array(),
			);

			$settings[ 'si_processor_limits_settings' . $class ]['settings'][ self::OPTION_PRE . $class . '_min' ] = array(
					'label' => sprintf( __( 'Minimum Payment Amount', 'sprout-invoices' ), $label ),
					'option' => array(
						'type' => 'text',
						'default' => self::get_payment_limit( $class ),
						'attributes' => array( 'class' => 'small-text' ),
						'description' => __( 'This payment processor will not show unless the payment total is at least this amount.' ),
					),
				);

			$settings[ 'si_processor_limits_settings' . $class ]['settings'][ self::OPTION_PRE . $class . '_max' ] = array(
					'label' => sprintf( __( 'Max Payment Amount', 'sprout-invoices' ), $label ),
					'option' => array(
						'type' => 'text',
						'default' => self::get_payment_limit( $class, false ),
						'attributes' => array( 'class' => 'small-text' ),
						'description' => __( 'The payment process will not be available if the payment total is over this amount. For no maximum use an extremely high number, e.g. 10000000.' ),
					),
				);

			if ( method_exists( $payment_processor, 'bank_supported' ) && $payment_processor->bank_supported() ) {

				$settings[ 'si_processor_limits_settings' . $class ]['settings'][ self::OPTION_PRE . $class . '_ach_min' ] = array(
						'label' => sprintf( __( 'Minimum ACH Payment Amount', 'sprout-invoices' ), $label ),
						'option' => array(
							'type' => 'text',
							'default' => self::get_payment_limit( $class, true, true ),
							'attributes' => array( 'class' => 'small-text' ),
							'description' => __( 'This payment processor will not show unless the payment total is at least this amount.' ),
						),
					);

				$settings[ 'si_processor_limits_settings' . $class ]['settings'][ self::OPTION_PRE . $class . '_ach_max' ] = array(
						'label' => sprintf( __( 'Max ACH Payment Amount', 'sprout-invoices' ), $label ),
						'option' => array(
							'type' => 'text',
							'default' => self::get_payment_limit( $class, false, true ),
							'attributes' => array( 'class' => 'small-text' ),
							'description' => __( 'The payment process will not be available if the payment total is over this amount. For no maximum use an extremely high number, e.g. 10000000.' ),
						),
					);
			}
		}
		return $settings;
	}

	/**
	 * Register the settings information. For the Processor Limits add-on, we need to link to the payment settings location.
	 * This data will be used by the add-ons page to link to payment settings.
	 *
	 * @param  array $settings The settings array.
	 * @return array $settings
	 */
	public static function register_addon_settings( $settings = array() ) {
		// Settings Page
		$settings['sprout-invoices-add-on-processor-limits-by-payment-amount'] = array(
			'title'  => __( 'Processor Limits', 'sprout-invoices' ),
			'weight' => 300,
			'tab'    => '',
			'link'   => admin_url( 'admin.php?page=sprout-invoices-payments' ),
		);
		return $settings;
	}

	public static function settings_label( $label = '' ) {
		return sprintf( __( '%s Limits', 'sprout-invoices' ), $label );
	}
}
