<?php

/**
 * SI_Service_Fee Controller
 *
 * @package Sprout_Invoice
 * @subpackage SI_Service_Fee
 */
class SI_Service_Fee_Settings extends SI_Service_Fee {

	public static function init() {

		// settings
		add_filter( 'si_payment_settings', array( __CLASS__, 'register_options' ) );
		add_filter( 'si_settings', array( __CLASS__, 'register_addon_settings' ) );
	}

	/**
	 * Hooked on init add the settings page and options.
	 *
	 */
	public static function register_options( $settings = array() ) {
		// Settings
		$settings['si_service_fee_settings'] = array(
				'title' => __( 'Service Fee Options', 'sprout-invoices' ),
				'weight' => 300,
				'settings' => array(),
			);

		$enabled_gateways = SI_Payment_Processors::enabled_processors();
		$payment_gateways = SI_Payment_Processors::get_registered_processors();

		if ( empty( $enabled_gateways ) ) {

			$settings['si_service_fee_settings']['description'] = sprintf(
				// translators: 1: break and enable processor button 2: v-on open 3: v-on close 4: v-bind open 5: v-bind element 6: v-bind value 7: button text 8: closing button tag.
				__( 'In order to use service fee add-on you will need to enable a Processor first.%1$s%2$s%3$s%4$s%5$s%6$s%7$s%8$s', 'sprout-invoices' ),
				'<br><a class="button button-primary" id="si_enable_processor" href="#payment-processors"',
				' v-on:click="makeTabActive(',
				"'payment-processors')",
				'" v-bind:class="{ active : isActiveTab(',
				"'payment-processors')",
				'== true }">',
				__( 'Enable a Processor', 'sprout-invoices' ),
				'</a>',
			);
		}
		foreach ( $payment_gateways as $class => $label ) {
			if ( ! in_array( $class, $enabled_gateways ) ) {
				continue;
			}

			if ( ! method_exists( $class, 'get_instance' ) ) {
				continue;
			}

			$payment_processor = call_user_func( array( $class, 'get_instance' ) );

			$settings['si_service_fee_settings']['settings'][ self::OPTION_PRE . $class ] = array(
					'label' => sprintf( __( '%s Service Fee', 'sprout-invoices' ), $label ),
					'option' => array(
						'type' => 'text',
						'default' => self::get_service_fee( $class ),
						'attributes' => array( 'class' => 'small-text' ),
						'description' => __( 'Percentage based on subtotal (before tax & discounts).' ),
					),
				);

			if ( method_exists( $payment_processor, 'bank_supported' ) && $payment_processor->bank_supported() ) {

				if ( 'SA_Stripe_Profiles' === $class || 'SA_Stripe' === $class  ) {
					$settings['si_service_fee_settings']['settings'][ self::OPTION_ACH_PRE . $class ] = array(
						'label' => sprintf( __( 'Plaid Service Fee', 'sprout-invoices' ), $label ),
						'option' => array(
							'type' => 'text',
							'default' => self::get_service_fee( $class, false ),
							'attributes' => array( 'class' => 'small-text', 'disabled' => 'disabled' ),
							'description' => __( 'Service fees for Plaid are not supported, instead the Stripe service fee is added.' ),
						),
					);
				} else {
					$settings['si_service_fee_settings']['settings'][ self::OPTION_ACH_PRE . $class ] = array(
						'label' => sprintf( __( '%s ACH Service Fee', 'sprout-invoices' ), $label ),
						'option' => array(
							'type' => 'text',
							'default' => self::get_service_fee( $class, true ),
							'attributes' => array( 'class' => 'small-text' ),
							'description' => __( 'Percentage based on subtotal (before tax & discounts).' ),
						),
					);
				}
			}
		}
		return $settings;
	}

	/**
	 * Register the settings information. For the Processor Limits add-on, we need to link to the payment settings location.
	 * This data will be used by the add-ons page to link to payment settings.
	 *
	 * @param  array $settings The settings array.
	 * @return array $settings
	 */
	public static function register_addon_settings( $settings = array() ) {
		// Settings Page
		$settings['sprout-invoices-add-on-service-convenience-fees-per-payment-processor'] = array(
			'title'  => __( 'Processor Service Fees', 'sprout-invoices' ),
			'weight' => 300,
			'tab' 	 => '',
			'link'   => admin_url( 'admin.php?page=sprout-invoices-payments' ),
		);
		return $settings;
	}
}
