<?php


/**
 * Sprout_Invoices Silent Payment Gateway
 *
 * Provides a Sprout_Invoices Payment Gateway, Used to create invoices regardless.
 *
 * @class 		Woo_Silent_Payments_Integration
 * @version		1.1.0
 * @package		WooCommerce/Classes/Payment
 * @author 		Elana D. 
 */


class Woo_Silent_Payments_Integration {
	const ORDER_ID_META = '_si_woocommerce_order';

	public static function init() {

            // HERE define you payment gateway ID (from $this->id from woo_invoice_payment.php)
            $payment_gateway_id = 'sprout_invoices';

            // Get an instance of the WC_Payment_Gateways object
            $payment_gateways   = WC_Payment_Gateways::instance();

            // Get the desired WC_Payment_Gateway object
            $payment_gateway    = $payment_gateways->payment_gateways()[$payment_gateway_id];

            // Display all the raw data for this payment gateway (Testing)
            //echo '<pre>'; print_r( $payment_gateway ); echo '</pre>'; 

            //Find out if Sprout Invoices payment processor is enabled in WooCommerce
            $settings_enabled = $payment_gateway->settings['enabled'];
            //Find out if the setting for Silent Invoices is enabled. 
            $settings_silent = $payment_gateway->settings['SilentInvoice'];
            if($settings_enabled === 'yes'){
                if($settings_silent === 'yes'){
                    add_action( 'woocommerce_order_status_processing', array( __CLASS__, 'SI_WooCommerce_Create_Silent_Invoice' ));
                }
            } 


	}

    public static function _Payment_Gateway_Disable_Sprout_checkout( $available_gateways ){
		//payment_method_sprout_invoices
        if ( isset( $available_gateways['sprout_invoices'] ) && current_user_can( 'manage_woocommerce' ) ) {
            unset( $available_gateways['sprout_invoices'] );
         } 
         return $available_gateways;
	}

    public static function SI_WooCommerce_Create_Silent_Invoice(  $order_id ) {

        $order = wc_get_order( $order_id );
        //echo '<script>alert('.$order.')</script>';

		///////////////////////////
		// Create Sprout Invoice //
		///////////////////////////

		/**
		 * Build line item array
		 * @var array
		 */
		$line_items = array();
		$tax = 0;
		foreach ( $order->get_items() as $key => $item ) {
			$_product  = apply_filters( 'woocommerce_order_item_product', $order->get_product_from_item( $item ), $item );
			if ( $_product && ! $_product->is_visible() ) {
				$desc = apply_filters( 'woocommerce_order_item_name', $item['name'], $item );
			} else {
				$desc = apply_filters( 'woocommerce_order_item_name', sprintf( '<a href="%s">%s</a>', get_permalink( $item['product_id'] ), $item['name'] ), $item );
			}
			$li_total = $item['line_total'];
			if ( $order->get_prices_include_tax() ) {
				$li_total = $item['line_total'] + $item['line_tax'];
			}
			$line_items[] = array(
				'rate' => $li_total / $item['qty'],
				'qty' => (int) $item['qty'],
				'desc' => $desc,
				'total' => $li_total,
				'tax' => $tax,
				'wc_item' => $item,
				);
		}
		$invoice_args = array(
			'status' => SI_Invoice::STATUS_PAID,
			'subject' => sprintf( __( 'Order #%s', 'sprout-invoices' ), $order_id ),
			'line_items' => $line_items,
		);

		$invoice_args = apply_filters( 'si_woo_process_payment_invoice_args', $invoice_args, $order_id );

		$invoice_id = SI_Invoice::create_invoice( $invoice_args );
		$invoice = SI_Invoice::get_instance( $invoice_id );

		$shipping = (float) $order->get_shipping_total();
		$total = (float) $order->get_total();

		// Set tax unless the tax it's already included
		if ( ! $order->get_prices_include_tax() ) {
			$tax = (float) 0;
			if ( ! empty( $order->get_tax_totals() )  ) {
				$tax_total = (float) 0;
				foreach ( $order->get_tax_totals() as $code => $tax ) {
					$tax_total += wc_round_tax_total( $tax->amount );
				}
				$tax = ( $tax_total / (float) $order->get_subtotal() ) * 100;
			}

			$invoice->set_tax( $tax );
		}

		$invoice->set_shipping( $shipping );
		$invoice->set_total( $total );

		if ( method_exists( $invoice, 'set_email' ) ) {
			$invoice->set_email( $order->billing_email );
		}

		$user_id = $order->get_user_id();

		// user does not exists, create user
		if ( ! $user_id ) {
			$email = $order->get_billing_email();

			$user_args = array(
				'user_login' => $order->get_billing_email(),
				'display_name' => $order->get_billing_company(),
				'user_pass' => wp_generate_password(), // random password
				'user_email' => $order->get_billing_email(),
				'first_name' => $order->get_billing_first_name(),
				'last_name' => $order->get_billing_last_name(),
			);
			$user_id = SI_Clients::create_user( $user_args );

			// obvi client id doesn't exist
			$client_ids = array();
		} // user exists
		else {
			$client_ids = SI_Client::get_clients_by_user( $user_id );
		}

		// get the first.
		if ( ! empty( $client_ids ) ) {
			$client_id = array_pop( $client_ids );
		}

		// create client if non exist
		if ( ! $client_id ) {
			$address = array(
				'street' => $order->billing_address_1 . ' ' . $order->billing_address_2,
				'city' => $order->billing_city,
				'zone' => $order->billing_state,
				'postal_code' => $order->billing_postcode,
				'country' => $order->billing_country,
			);

			$args = array(
				'company_name' => ( '' !== $order->billing_company ) ? $order->billing_company : $order->billing_first_name . ' ' . $order->billing_last_name,
				'website' => '',
				'address' => $address,
				'user_id' => $user_id,
			);
			$client_id = SI_Client::new_client( $args );
		}

		$invoice->set_client_id( $client_id );

		// History
		do_action( 'si_new_record',
			sprintf( '<a href="%s">#%s</a>', add_query_arg( array( 'post' => $order_id, 'action' => 'edit' ), admin_url( 'post.php' ) ), $order_id ),
			'woocommerce_order',
			$invoice_id,
			sprintf( '<a href="%s">#%s</a>', add_query_arg( array( 'post' => $order_id, 'action' => 'edit' ), admin_url( 'post.php' ) ), $order_id ),
			0,
		false );

		// store the order id within the invoice
		update_post_meta( $invoice_id, Woo_Payments_Integration::ORDER_ID_META, $order_id );
    
        
    }

}