<?php

/**
 *
 * @package SI
 * @subpackage Payment SI_Offsite_Processors
 */
class SA_Cash extends SI_Offsite_Processors {
	const MODE_LIVE = 'production';
	const CASHTAGOPTION = 'si_sc_cash';
	const PAYMENT_METHOD = 'Cash';
	const PAYMENT_SLUG = 'cash';

	protected static $instance;
	private static $cashtag = '';

	public static function get_instance() {
		if ( ! ( isset( self::$instance ) && is_a( self::$instance, __CLASS__ ) ) ) {
			self::$instance = new self();
		}
		return self::$instance;
	}

	public function get_payment_method() {
		return self::PAYMENT_METHOD;
	}

	public function get_slug() {
		return self::PAYMENT_SLUG;
	}

	public static function register() {
		self::add_payment_processor( __CLASS__, __( 'Cash' , 'sprout-invoices' ) );
	}


	public static function public_name() {
		return __( 'Cash' , 'sprout-invoices' );
	}

	public static function checkout_options() {
		$option = array(
			'icons' => array(
				SI_URL . '/resources/front-end/img/dollar.png',
				),
			'label' => __( 'Cash' , 'sprout-invoices' ),
			);
		return apply_filters( 'si_cash_checkout_options', $option );
	}

	protected function __construct() {
		parent::__construct();
		self::$cashtag = get_option( self::CASHTAGOPTION, '' );

		add_action( 'si_checkout_action_'.SI_Checkouts::PAYMENT_PAGE, array( $this, 'send_offsite' ), 0, 1 );

		// Remove pages
		add_filter( 'si_checkout_pages', array( $this, 'remove_checkout_pages' ) );
	}

	/**
	 * The review page is unnecessary
	 *
	 * @param array   $pages
	 * @return array
	 */
	public function remove_checkout_pages( $pages ) {
		unset( $pages[ SI_Checkouts::REVIEW_PAGE ] );
		return $pages;
	}

	/**
	 * Hooked on init add the settings page and options.
	 *
	 */
	public static function register_settings( $settings = array() ) {
		// Settings
		$settings['payments'] = array(
			'si_cash_settings' => array(
					'title' => __( 'Custom Payment URLs' , 'sprout-invoices' ),
					'description' => __( 'There are no settings for Cash Payment.' , 'sprout-invoices' ),
					'weight' => 200,
				),
			);
		return $settings;
	}

	/**
	 * Instead of redirecting to the SIcheckout page,
	 * set up the Express Checkout transaction and redirect there
	 *
	 * @param SI_Carts $cart
	 * @return void
	 */
	public function send_offsite( SI_Checkouts $checkout ) {
		// Check to see if the payment processor being used is for this payment processor
		if ( ! is_a( $checkout->get_processor(), __CLASS__ ) ) { 
			return;
		}
		$invoice = $checkout->get_invoice();
		$invoice->set_as_paid();

	}

	/**
	 * Process a payment
	 *
	 * @param SI_Checkouts $checkout
	 * @param SI_Invoice $invoice
	 * @return SI_Payment|bool false if the payment failed, otherwise a Payment object
	 */
	public function process_payment( SI_Checkouts $checkout, SI_Invoice $invoice ) {
		// Nothing, since the user is not returned.
	}
}
SA_Cash::register();
